@@ -13,7 +13,7 @@
 
 package IO::Socket::SSL;
 
-our $VERSION = '2.032';
+our $VERSION = '2.033';
 
 use IO::Socket;
 use Net::SSLeay 1.46;
@@ -30,9 +30,15 @@ BEGIN {
 }
 
 
+# results from commonly used constant functions from Net::SSLeay for fast access
+my $Net_SSLeay_ERROR_WANT_READ   = Net::SSLeay::ERROR_WANT_READ();
+my $Net_SSLeay_ERROR_WANT_WRITE  = Net::SSLeay::ERROR_WANT_WRITE();
+my $Net_SSLeay_VERIFY_NONE       = Net::SSLeay::VERIFY_NONE();
+my $Net_SSLeay_VERIFY_PEER       = Net::SSLeay::VERIFY_PEER();
 
-use constant SSL_VERIFY_NONE => Net::SSLeay::VERIFY_NONE();
-use constant SSL_VERIFY_PEER => Net::SSLeay::VERIFY_PEER();
+
+use constant SSL_VERIFY_NONE => &Net::SSLeay::VERIFY_NONE;
+use constant SSL_VERIFY_PEER => &Net::SSLeay::VERIFY_PEER;
 use constant SSL_VERIFY_FAIL_IF_NO_PEER_CERT => Net::SSLeay::VERIFY_FAIL_IF_NO_PEER_CERT();
 use constant SSL_VERIFY_CLIENT_ONCE => Net::SSLeay::VERIFY_CLIENT_ONCE();
 
@@ -54,6 +60,7 @@ my $can_alpn;        # do we support ALPN
 my $can_ecdh;        # do we support ECDH key exchange
 my $can_ocsp;        # do we support OCSP
 my $can_ocsp_staple; # do we support OCSP stapling
+my $can_tckt_keycb;  # TLS ticket key callback
 BEGIN {
     $can_client_sni = Net::SSLeay::OPENSSL_VERSION_NUMBER() >= 0x01000000;
     $can_server_sni = defined &Net::SSLeay::get_servername;
@@ -67,6 +74,7 @@ BEGIN {
     $can_ocsp        = defined &Net::SSLeay::OCSP_cert2ids;
     $can_ocsp_staple = $can_ocsp
 	&& defined &Net::SSLeay::set_tlsext_status_type;
+    $can_tckt_keycb  = defined &Net::SSLeay::CTX_set_tlsext_ticket_getkey_cb;
 }
 
 my $algo2digest = do {
@@ -266,9 +274,9 @@ use vars qw(@ISA $SSL_ERROR @EXPORT);
     # the usual way of problems. Like with EWOULDBLOCK, EINPROGRESS..
     # these are especially important for non-blocking sockets
 
-    my $x = Net::SSLeay::ERROR_WANT_READ();
+    my $x = $Net_SSLeay_ERROR_WANT_READ;
     use constant SSL_WANT_READ  => dualvar( \$x, 'SSL wants a read first' );
-    my $y = Net::SSLeay::ERROR_WANT_WRITE();
+    my $y = $Net_SSLeay_ERROR_WANT_WRITE;
     use constant SSL_WANT_WRITE => dualvar( \$y, 'SSL wants a write first' );
 
     @EXPORT = qw(
@@ -608,9 +616,9 @@ sub configure_SSL {
 sub _skip_rw_error {
     my ($self,$ssl,$rv) = @_;
     my $err = Net::SSLeay::get_error($ssl,$rv);
-    if ( $err == Net::SSLeay::ERROR_WANT_READ()) {
+    if ( $err == $Net_SSLeay_ERROR_WANT_READ) {
 	$SSL_ERROR = SSL_WANT_READ;
-    } elsif ( $err == Net::SSLeay::ERROR_WANT_WRITE()) {
+    } elsif ( $err == $Net_SSLeay_ERROR_WANT_WRITE) {
 	$SSL_ERROR = SSL_WANT_WRITE;
     } else {
 	return $err;
@@ -1045,7 +1053,7 @@ sub accept_SSL {
 
 sub _generic_read {
     my ($self, $read_func, undef, $length, $offset) = @_;
-    my $ssl = $self->_get_ssl_object || return;
+    my $ssl =  ${*$self}{_SSL_object} || return;
     my $buffer=\$_[2];
 
     $SSL_ERROR = $! = undef;
@@ -1106,7 +1114,7 @@ sub peek {
 sub _generic_write {
     my ($self, $write_all, undef, $length, $offset) = @_;
 
-    my $ssl = $self->_get_ssl_object || return;
+    my $ssl =  ${*$self}{_SSL_object} || return;
     my $buffer = \$_[2];
 
     my $buf_len = length($$buffer);
@@ -1358,9 +1366,9 @@ sub stop_SSL {
 		    last if $wait<=0;
 		    vec(my $vec = '',fileno($self),1) = 1;
 		    my $err = Net::SSLeay::get_error($ssl,$rv);
-		    if ( $err == Net::SSLeay::ERROR_WANT_READ()) {
+		    if ( $err == $Net_SSLeay_ERROR_WANT_READ) {
 			select($vec,undef,undef,$wait)
-		    } elsif ( $err == Net::SSLeay::ERROR_WANT_READ()) {
+		    } elsif ( $err == $Net_SSLeay_ERROR_WANT_READ) {
 			select(undef,$vec,undef,$wait)
 		    } else {
 			last;
@@ -2128,6 +2136,10 @@ use constant SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER => 2;
 use constant FILETYPE_PEM => Net::SSLeay::FILETYPE_PEM();
 use constant FILETYPE_ASN1 => Net::SSLeay::FILETYPE_ASN1();
 
+my $DEFAULT_SSL_OP = &Net::SSLeay::OP_ALL
+    | &Net::SSLeay::OP_SINGLE_DH_USE
+    | ($can_ecdh && &Net::SSLeay::OP_SINGLE_ECDH_USE);
+
 # Note that the final object will actually be a reference to the scalar
 # (C-style pointer) returned by Net::SSLeay::CTX_*_new() so that
 # it can be blessed.
@@ -2183,7 +2195,7 @@ sub new {
     }
 
     my $verify_mode = $arg_hash->{SSL_verify_mode} || 0;
-    if ( $verify_mode != Net::SSLeay::VERIFY_NONE()) {
+    if ( $verify_mode != $Net_SSLeay_VERIFY_NONE) {
 	for (qw(SSL_ca_file SSL_ca_path)) {
 	    $CHECK_SSL_PATH->($_ => $arg_hash->{$_} || next);
 	}
@@ -2239,9 +2251,15 @@ sub new {
 	};
     }
 
-    my $ssl_op = Net::SSLeay::OP_ALL();
-    $ssl_op |= &Net::SSLeay::OP_SINGLE_DH_USE;
-    $ssl_op |= &Net::SSLeay::OP_SINGLE_ECDH_USE if $can_ecdh;
+    if ($is_server) {
+	if ($arg_hash->{SSL_ticket_keycb} && !$can_tckt_keycb) {
+	    warn "Ticket Key Callback is not supported - ignoring option SSL_ticket_keycb\n";
+	    delete $arg_hash->{SSL_ticket_keycb};
+	}
+    }
+
+
+    my $ssl_op = $DEFAULT_SSL_OP;
 
     my $ver;
     for (split(/\s*:\s*/,$arg_hash->{SSL_version})) {
@@ -2271,7 +2289,7 @@ sub new {
 
     # For SNI in server mode we need a separate context for each certificate.
     my %ctx;
-    if ($arg_hash->{'SSL_server'}) {
+    if ($is_server) {
 	my %sni;
 	for my $opt (qw(SSL_key SSL_key_file SSL_cert SSL_cert_file)) {
 	    my $val  = $arg_hash->{$opt} or next;
@@ -2345,6 +2363,12 @@ sub new {
 	    }
 	}
 
+	if ($arg_hash->{SSL_ticket_keycb}) {
+	    my $cb = $arg_hash->{SSL_ticket_keycb};
+	    ($cb,my $arg) = ref($cb) eq 'CODE' ? ($cb):@$cb;
+	    Net::SSLeay::CTX_set_tlsext_ticket_getkey_cb($ctx,$cb,$arg);
+	}
+
 	# Try to apply SSL_ca even if SSL_verify_mode is 0, so that they can be
 	# used to verify OCSP responses.
 	# If applying fails complain only if verify_mode != VERIFY_NONE.
@@ -2368,7 +2392,7 @@ sub new {
 		$ctx, $file || '', $dir || '')) {
 		return IO::Socket::SSL->error(
 		    "Invalid certificate authority locations")
-		    if $verify_mode != Net::SSLeay::VERIFY_NONE();
+		    if $verify_mode != $Net_SSLeay_VERIFY_NONE;
 	    }
 	} elsif ( my %ca = IO::Socket::SSL::default_ca()) {
 	    # no CA path given, continue with system defaults
@@ -2376,14 +2400,13 @@ sub new {
 	    $dir = join($OPENSSL_LIST_SEPARATOR,@$dir) if ref($dir);
 	    if (! Net::SSLeay::CTX_load_verify_locations( $ctx,
 		$ca{SSL_ca_file} || '',$dir || '')
-		&& $verify_mode != Net::SSLeay::VERIFY_NONE()) {
+		&& $verify_mode != $Net_SSLeay_VERIFY_NONE) {
 		return IO::Socket::SSL->error(
 		    "Invalid default certificate authority locations")
 	    }
 	}
 
-	if ($arg_hash->{SSL_server}
-	    && ($verify_mode & Net::SSLeay::VERIFY_PEER())) {
+	if ($is_server && ($verify_mode & $Net_SSLeay_VERIFY_PEER)) {
 	    if ($arg_hash->{SSL_client_ca}) {
 		for (@{$arg_hash->{SSL_client_ca}}) {
 		    return IO::Socket::SSL->error(
